@@ -60,13 +60,13 @@ class System:
 
     @staticmethod
     def _reflink_linux(src, dst):
-        import fcntl
+        import fcntl  # pylint: disable=import-error
 
         FICLONE = 0x40049409
 
         try:
             ret = 255
-            with open(src) as s, open(dst, "w+") as d:
+            with open(src, "rb") as s, open(dst, "wb+") as d:
                 ret = fcntl.ioctl(d.fileno(), FICLONE, s.fileno())
         finally:
             if ret != 0:
